import * as path from "jsr:@std/path";

const root = Deno.cwd();

const getAllFiles = async (dir: string) => {
	const localEntries = await Array.fromAsync(Deno.readDir(dir));
	const entries = localEntries.filter((e) => e.isFile).map((e) => e.name);
	for (const entry of localEntries) {
		if (
			entry.isDirectory &&
			!entry.isSymlink &&
			!entry.name.startsWith(".")
		) {
			entries.push(
				...(await getAllFiles(entry.name)).map((fname) =>
					path.join(entry.name, fname)
				)
			);
		}
	}
	return entries;
};

const getFilesToGenFrom = async () => {
	const allFiles = await getAllFiles(root);
	return allFiles
		.filter(
			(fname) => fname.endsWith(".gen.ts") || fname.endsWith(".gen.js")
		)
		.map((fname) => path.join(root, fname));
};

const filesBeingGenned: Record<string, Promise<void>> = {};

const genFromFile = async (fname: string) => {
	const context = {
		async import(moduleName: string) {
			const fullname = path.resolve(path.dirname(fname), moduleName);
			await filesBeingGenned[fullname];
			return import(fullname);
		},
	};
	type CodegenContext = typeof context;
	const brain: (context: CodegenContext) => string | Promise<string> = (
		await import(fname)
	).default;
	const fileToWrite = fname.replace(/\.gen\.[tj]sx?$/, (ext) =>
		ext.slice(".gen".length)
	);
	Deno.writeTextFile(fileToWrite, await brain(context));
};

for (const file of await getFilesToGenFrom()) {
	filesBeingGenned[file] = genFromFile(file);
}
